@@ -1,10 +1,11 @@
-__version__ = "0.1.8"
+__version__ = "0.2.0"
+
+from .distillers import BasicTrainer
+from .distillers import BasicDistiller
+from .distillers import GeneralDistiller
+from .distillers import MultiTaskDistiller
+from .distillers import MultiTeacherDistiller
 
-from .distillation import BasicTrainer
-from .distillation import BasicDistiller
-from .distillation import GeneralDistiller
-from .distillation import MultiTeacherDistiller
-from .distillation import MultiTaskDistiller
 
 from .configurations import TrainingConfig, DistillationConfig
 
@@ -20,4 +21,4 @@ Distillers = {
     'MultiTeacher': MultiTeacherDistiller,
     'MultiTask': MultiTaskDistiller,
     'Train': BasicTrainer
-}
\ No newline at end of file
+}
